@@ -24,6 +24,7 @@ int csum_length=SUM_LENGTH;
 #define CSUM_CHUNK 64
 
 int checksum_seed = 0;
+extern int remote_version;
 
 /*
   a simple 32 bit checksum that can be upadted from either end
@@ -83,7 +84,8 @@ void get_checksum2(char *buf,int len,char *sum)
   for(i = 0; i + CSUM_CHUNK <= len; i += CSUM_CHUNK) {
     MDupdate(&MD, buf1+i, CSUM_CHUNK*8);
   }
-  MDupdate(&MD, buf1+i, (len-i)*8);
+  if (len - i > 0)
+    MDupdate(&MD, buf1+i, (len-i)*8);
 
   sum_put(&MD,sum);
 }
@@ -93,7 +95,7 @@ void file_checksum(char *fname,char *sum,off_t size)
 {
   int i;
   MDstruct MD;
-  char *buf;
+  struct map_struct *buf;
   int fd;
   int len = size;
   char tmpchunk[CSUM_CHUNK];
@@ -112,37 +114,80 @@ void file_checksum(char *fname,char *sum,off_t size)
     MDupdate(&MD, tmpchunk, CSUM_CHUNK*8);
   }
 
-  bcopy(map_ptr(buf,i,len-i),tmpchunk,len-i);
-  MDupdate(&MD, tmpchunk, (len-i)*8);
+  if (len - i > 0) {
+    bcopy(map_ptr(buf,i,len-i),tmpchunk,len-i);
+    MDupdate(&MD, tmpchunk, (len-i)*8);
+  }
 
   sum_put(&MD,sum);
 
   close(fd);
-  unmap_file(buf,size);
+  unmap_file(buf);
 }
 
 
 void checksum_init(void)
 {
+  if (remote_version >= 14)
+    csum_length = 2; /* adaptive */
+  else
+    csum_length = SUM_LENGTH;
 }
 
 
-#ifdef CHECKSUM_MAIN
- int main(int argc,char *argv[])
+
+static MDstruct sumMD;
+static int sumresidue;
+static char sumrbuf[CSUM_CHUNK];
+
+void sum_init(void)
 {
-  char sum[SUM_LENGTH];
-  int i,j;
+  char s[4];
+  MDbegin(&sumMD);  
+  sumresidue=0;
+  SIVAL(s,0,checksum_seed);
+  sum_update(s,4);
+}
 
-  checksum_init();
+void sum_update(char *p,int len)
+{
+  int i;
+  if (len + sumresidue < CSUM_CHUNK) {
+    bcopy(p,sumrbuf+sumresidue,len);
+    sumresidue += len;
+    return;
+  }
 
-  for (i=1;i<argc;i++) {
-    struct stat st;
-    if (stat(argv[i],&st) == 0) {
-      file_checksum(argv[i],sum,st.st_size);
-      for (j=0;j<SUM_LENGTH;j++)
-	printf("%02X",(unsigned char)sum[j]);
-      printf("  %s\n",argv[i]);
-    }
+  if (sumresidue) {
+    i = MIN(CSUM_CHUNK-sumresidue,len);
+    bcopy(p,sumrbuf+sumresidue,i);
+    MDupdate(&sumMD, sumrbuf, (i+sumresidue)*8);
+    len -= i;
+    p += i;
+  }
+
+  for(i = 0; i + CSUM_CHUNK <= len; i += CSUM_CHUNK) {
+    bcopy(p+i,sumrbuf,CSUM_CHUNK);
+    MDupdate(&sumMD, sumrbuf, CSUM_CHUNK*8);
+  }
+
+  if (len - i > 0) {
+    sumresidue = len-i;
+    bcopy(p+i,sumrbuf,sumresidue);
+  } else {
+    sumresidue = 0;    
   }
 }
-#endif
+
+void sum_end(char *sum)
+{
+  if (sumresidue)
+    MDupdate(&sumMD, sumrbuf, sumresidue*8);
+
+  SIVAL(sum,0,sumMD.buffer[0]);
+  SIVAL(sum,4,sumMD.buffer[1]);
+  SIVAL(sum,8,sumMD.buffer[2]);
+  SIVAL(sum,12,sumMD.buffer[3]);  
+}
+
+
